/*!
 *
 * Copyright (C) 2007 Technical University of Liberec.  All rights reserved.
 *
 * Please make a following refer to Flow123d on your project site if you use the program for any purpose,
 * especially for academic research:
 * Flow123d, Research Centre: Advanced Remedial Technologies, Technical University of Liberec, Czech Republic
 *
 * This program is free software; you can redistribute it and/or modify it under the terms
 * of the GNU General Public License version 3 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program; if not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 021110-1307, USA.
 *
 *
 * $Id: quadrature_lib.hh 1690 2012-05-21 13:17:31Z jan.stebel $
 * $Revision: 1690 $
 * $LastChangedBy: jan.stebel $
 * $LastChangedDate: 2012-05-21 15:17:31 +0200 (Mon, 21 May 2012) $
 *
 * @file
 * @brief Definitions of particular quadrature rules on simplices.
 *  @author Jan Stebel
 */

#ifndef QUADRATURE_LIB_HH_
#define QUADRATURE_LIB_HH_

#include "quadrature/quadrature.hh"


/**
 * @brief Symmetric Gauss-Legendre quadrature formulae on simplices.
 *
 * Symmetric Gauss-Legendre quadrature on a @p dim dimensional simplex.
 * The coefficients are taken from Parallel Hierarchical Grid project.
 *
 */
template<unsigned int dim>
class QGauss : public Quadrature<dim> {
public:
    /**
     * @brief Create a formula of given order.
     *
     * The formula is exact for polynomials of degree @p order.
     */
    QGauss(const unsigned int order);
};





#endif /* QUADRATURE_LIB_HH_ */
